import * as fs from 'fs-extra';
import {
  getNormalizedAcceleratorConfig,
} from '../src/configuration/getNormalizedAcceleratorConfig';
import * as path from 'path';

// Make sure any symlinks in the project folder are resolved:
// https://github.com/facebook/create-react-app/issues/637
const appDirectory = fs.realpathSync(process.cwd());
const resolveApp = (relativePath) => path.resolve(appDirectory, relativePath);

const { publicUrl } = getNormalizedAcceleratorConfig();

export const paths = {
  publicUrl,
  moduleFileExtensions: [
    'js',
    'jsx',
    'mjs',
    'ts',
    'tsx',
    'json',
  ],

  appBuild: resolveApp('build-web/'),
  appIndex: resolveApp('src/index.tsx'),
  appNodeModules: resolveApp('node_modules/'),
  appPackageJson: resolveApp('package.json'),
  appPath: resolveApp('.'),
  appPublic: resolveApp('public'),
  appSrc: resolveApp('src/'),
  appTsConfig: resolveApp('tsconfig.json'),
  acceleratorConfig: resolveApp('accelerator.config.js'),
  fontAutogeneratedStyle: resolveApp('public/fonts/autogenerated/fontface-autogen.css'),
  fontFaceObserverTemplate: resolveApp('templates/fontFaceObserver.hbs'),
  fontsAutogenerationDir: resolveApp('public/fonts/autogenerated/'),
  fontsDir: resolveApp('public/fonts/'),
  fontLoaderTemplate: resolveApp('templates/fontLoader.hbs'),
  footers: resolveApp('footers/'),
  headers: resolveApp('headers/'),
  htmlTemplate: resolveApp('templates/index.hbs'),
  indexHtml: resolveApp('build-web/index.html'),
  lib: resolveApp('lib'),
  passages: resolveApp('passages/'),
  plugins: resolveApp('plugins/'),
  proxySetup: resolveApp('src/setupProxy.ts'),
  publicDir: resolveApp('public/'),
  renderers: resolveApp('renderers/'),
  storyOptions: resolveApp('options/'),
  testsSetup: resolveApp('src/setupTests.ts'),
};
