import {
  error,
} from 'colorful-logging';
import * as fs from 'fs-extra';
import {
  getAuthoredAssetObjectDefinitions,
} from './functions/getAuthoredAssetObjectDefinitions';
import {
  getAutogeneratedFileWarning,
} from './functions/getAutogeneratedFileWarning';
import {
  getFileImports,
} from './functions/getFileImports';
import {
  getHotReloadAcceptor,
} from './functions/getHotReloadAcceptor';
import * as path from 'path';
import {
  setUnhandledRejectionEvent,
} from './functions/setUnhandledRejectionEvent';
import {
  scrapeAssets,
} from './functions/scrapeAssets';

setUnhandledRejectionEvent();

const authoredFootersDir = path.join(__dirname, '..', 'footers');

/* Collect all files within the footers directory ending in .js, .jsx, .ts, or .tsx. */
(async () => {
  try {
    const files = await scrapeAssets(authoredFootersDir);

    const {
      importPaths,
      imports,
    } = getFileImports(files);

    const manifestStr =
      getAutogeneratedFileWarning() +
      '\n\nimport { IFooterManifestItem } from \'../src/passages/IFooterManifestItem\';\n\n' +
      imports.join('\n') +
      '\nconst manifest: ReadonlyArray<IFooterManifestItem> = [\n' +
      getAuthoredAssetObjectDefinitions(files).join('\n') +
      '\n];\n\nexport default manifest;\n\n' +
      getHotReloadAcceptor(importPaths) +
      '\n';

    const manifestPath = path.join(authoredFootersDir, 'footers-manifest.ts');
    await fs.writeFile(manifestPath, manifestStr);
  } catch (err) {
    error(err);
    process.exit(1);
  }
})();
