import {
  error,
} from 'colorful-logging';
import * as fs from 'fs-extra';
import {
  getAutogeneratedFileWarning,
} from './functions/getAutogeneratedFileWarning';
import {
  getFileImports,
} from './functions/getFileImports';
import {
  getHotReloadAcceptor,
} from './functions/getHotReloadAcceptor';
import {
  getAuthoredAssetObjectDefinitions,
} from './functions/getAuthoredAssetObjectDefinitions';
import * as path from 'path';
import {
  setUnhandledRejectionEvent,
} from './functions/setUnhandledRejectionEvent';
import {
  scrapeAssets,
} from './functions/scrapeAssets';

setUnhandledRejectionEvent();

const authoredHeadersDir = path.join(__dirname, '..', 'headers');

(async () => {
  try {
    const files = await scrapeAssets(authoredHeadersDir);

    const {
      importPaths,
      imports,
    } = getFileImports(files);

    const manifestStr =
      getAutogeneratedFileWarning() +
      '\n\nimport { IHeaderManifestItem } from \'../src/passages/IHeaderManifestItem\';\n\n' +
      imports.join('\n') +
      '\nconst manifest: ReadonlyArray<IHeaderManifestItem> = [\n' +
      getAuthoredAssetObjectDefinitions(files).join('\n') +
      '\n];\n\nexport default manifest;\n' +
      getHotReloadAcceptor(importPaths) +
      '\n';

    const manifestPath = path.join(authoredHeadersDir, 'headers-manifest.ts');
    await fs.writeFile(manifestPath, manifestStr);
  } catch (err) {
    error(err);
    process.exit(1);
  }
})();
