import {
  error,
} from 'colorful-logging';
import * as fs from 'fs-extra';
import {
  getAutogeneratedFileWarning,
} from './functions/getAutogeneratedFileWarning';
import {
  getFileImports,
} from './functions/getFileImports';
import {
  getHotReloadAcceptor,
} from './functions/getHotReloadAcceptor';
import {
  getAuthoredAssetObjectDefinitions,
} from './functions/getAuthoredAssetObjectDefinitions';
import * as path from 'path';
import {
  scrapeAssets,
} from './functions/scrapeAssets';
import {
  setUnhandledRejectionEvent,
} from './functions/setUnhandledRejectionEvent';

setUnhandledRejectionEvent();

const authoredPassagesDir = path.join(__dirname, '..', 'passages');

(async () => {
  try {
    /* Do not include the init file as a passage. */
    const files = await scrapeAssets(authoredPassagesDir);

    const {
      importPaths,
      imports,
    } = getFileImports(files);
  
    const manifestStr =
      getAutogeneratedFileWarning() +
      '\n\nimport { IPassageManifestItem } from \'../src/passages/IPassageManifestItem\';\n\n' +
      imports.join('\n') +
      '\nconst manifest: IPassageManifestItem[] = [\n' +
      getAuthoredAssetObjectDefinitions(files).join('\n') +
      '\n];\n\nexport default manifest;\n\n' +
      getHotReloadAcceptor(importPaths) +
      '\n';

    const manifestPath = path.join(authoredPassagesDir, 'passages-manifest.ts');
    await fs.writeFile(manifestPath, manifestStr);
  } catch (err) {
    error(err);
    process.exit(1);
  }
})();
