import {
  error,
} from 'colorful-logging';
import * as fs from 'fs-extra';
import {
  getAutogeneratedFileWarning,
} from './functions/getAutogeneratedFileWarning';
import {
  getAuthoredAssetObjectDefinitions,
} from './functions/getAuthoredAssetObjectDefinitions';
import {
  getFileImports,
} from './functions/getFileImports';
import {
  getHotReloadAcceptor,
} from './functions/getHotReloadAcceptor';
import * as path from 'path';
import {
  scrapeAssets,
} from './functions/scrapeAssets';
import {
  setUnhandledRejectionEvent,
} from './functions/setUnhandledRejectionEvent';

setUnhandledRejectionEvent();

const authoredStoryOptionsDir = path.join(__dirname, '..', 'options');

(async () => {
  try {
    const files = await scrapeAssets(authoredStoryOptionsDir);

    const {
      importPaths,
      imports,
    } = getFileImports(files);
  
    const manifestStr =
      getAutogeneratedFileWarning() +
      '\nimport { IStoryOptionManifestItem } from \'../src/storyOptions/IStoryOptionManifestItem\';\n\n' +
      imports.join('\n') +
      '\n\nconst manifest: ReadonlyArray<IStoryOptionManifestItem> = [\n' +
      getAuthoredAssetObjectDefinitions(files).join('\n') +
      '\n];\n\n' +
      'export default manifest;\n\n' + 
      getHotReloadAcceptor(importPaths) +
      '\n';

    const manifestPath = path.join(authoredStoryOptionsDir, 'storyOptions-manifest.ts');
    await fs.writeFile(manifestPath, manifestStr);
  } catch (err) {
    error(err);
    process.exit(1);
  }
})();
